/* String utility routines */   /*jcr16jul87*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/param.h>
#include "son.h"

char *Skip_blnks(Str)
char *Str;
/*************************************************************/
/* Return a pointer to the first non-blank char in Str.      */
/* (C) COPYRIGHT 1986 JAMES C. RAUTIO.  ALL RIGHTS RESERVED. */
/*************************************************************/
/* jcr31may87 */
{
	while (isspace(*Str) || *Str == ',' )
		Str++;
	return (Str);
}			/*Skip_blnks*/

char *Next_field(Str)
char *Str;
/*************************************************************/
/* Return a pointer to the first char of the next field.     */
/* (C) COPYRIGHT 1986 JAMES C. RAUTIO.  ALL RIGHTS RESERVED. */
/*************************************************************/
/* jcr31may87 */
{
	while (! isspace(*Str) && *Str != ',' && *Str != '\0')
		Str++;
	return (Skip_blnks(Str));
}			/*Next_field*/

/*Set to -1 so sFgets can detect user not having reset line count*/
static int Fgets_line_count = -1;

void
Fgets_done()
/*************************************************************/
/* User has finished getting lines from a file, set line     */
/* count to be -1 					     */
/* (C) COPYRIGHT 1991 JAMES C. RAUTIO.  ALL RIGHTS RESERVED. */
/*************************************************************/
/* jcr17apr91 */
{
	Fgets_line_count = -1;
}

void
Fgets_reset()
/*************************************************************/
/* Reset line count for Fgets.                               */
/* (C) COPYRIGHT 1991 JAMES C. RAUTIO.  ALL RIGHTS RESERVED. */
/*************************************************************/
/* jcr17apr91 */
{
	Fgets_line_count = 0;
}

Fgets_line_num()
/*************************************************************/
/* Reset line count for Fgets.                               */
/* (C) COPYRIGHT 1991 JAMES C. RAUTIO.  ALL RIGHTS RESERVED. */
/*************************************************************/
/* jcr17apr91 */
{
	return (Fgets_line_count);
}

char *sFgets(s, n, in)
char *s;
int n;
FILE *in;
/*************************************************************/
/* Get str from in, rm nl, count lines.                      */
/* (C) COPYRIGHT 1991 JAMES C. RAUTIO.  ALL RIGHTS RESERVED. */
/*************************************************************/
/* jcr17apr91 */
{
	char *rtrn;
	int i;

	if (Fgets_line_count < 0)	/*line count was not initialized*/
		announce_bug("Fgets");

	rtrn = fgets(s, n, in);
	if (rtrn == NULL)
		return (rtrn);
  /*Change new line, \012, to \000*/
	i = 0;
	while (i < n && *(s + i) != '\000' && *(s + i) != '\012')
		i++;
	if (i >= n)
		i = n - 1;
	*(s + i) = '\000';
	Fgets_line_count++;
	return (s);
}			/*sFgets*/


/*******************************************************************
	NAME: get_sonnet_dir
	
	DESCRIPTION: Get the SONNET_DIR variable

		check if SONNET_DIR variable is set.  If so return
		a char point to the variable.  If not print out an
		error message.

	RETURNS: 
		TRUE - was able to get SONNET_DIR variable
		NULL - was not able to get SONNET_DIR variable

*******************************************************************/

char *
get_sonnet_dir(print_error)
int print_error;	/* If TRUE print out error message if SONNET_DIR */
			/* is not defined				 */
{
	char *sonnet_dir;
	extern char *getenv();

	sonnet_dir = getenv("SONNET_DIR");
#ifdef DOS
	if (sonnet_dir == NULL) {
		sonnet_dir = "c:/sonnet";
	}
	else if (strcmp(sonnet_dir, "") == 0) {
		sonnet_dir = "c:/sonnet";
	}
#endif
	if (sonnet_dir == NULL) {
		if (print_error) {
			fprintf(stderr, "The variable SONNET_DIR must be defined.\n");
			fprintf(stderr, "You can use the Sonnet Software command `sonnetsetup`\n");
			fprintf(stderr, "to setup your login.  To run this command, type in the\n");
			fprintf(stderr, "following:\n");
			fprintf(stderr, "\n");
			fprintf(stderr, "           cd <the sonnet software directory>\n");
			fprintf(stderr, "           ./sonnetsetup\n");
			fprintf(stderr, "\n");
			fprintf(stderr, "If you do not know where the Sonnet Software directory is,\n");
			fprintf(stderr, "see the person who did the Sonnet Software installation.\n");
		}
		return (sonnet_dir);
	}

	if (strcmp(sonnet_dir, "") == 0) {
		if (print_error) {
			fprintf(stderr, "The variable SONNET_DIR is set but is empty\n");
		}
		return (NULL);
	}
	return (sonnet_dir);
}

/*******************************************************************
	NAME: rmspace
	
	DESCRIPTION: Remove leading spaces from string "strng".  
		     Puts results back into the same string.
	
	RETURNS: strng - with spaces removed.
*******************************************************************/

#undef SPACE
#define SPACE ' '

void
rmspace(strng)
char *strng;
{
	int c, i;

	/* find first non-blank string */
	c = 0;
	while (strng[c] == SPACE) {
		++c;
	}

	/* move string to get rid of blanks */
	i = 0;
	while (strng[c] != 0) {
		strng[i++] = strng[c++];
	}

	/* terminates string with a null */
	strng[i] = 0;
}


/*******************************************************************
	NAME: file_delimeter
	
	DESCRIPTION: Returns the file delimiter for the given computer.
	
	RETURNS:
		return "/" for UNIX
		return "\" for DOS
*******************************************************************/

char *
file_delimeter(){
#ifdef DOS
	return ("\\");
#else
	return ("/");
#endif
}


/*******************************************************************
	NAME: filename_cat
	
	DESCRIPTION: append a path/file name to a base file name
		Assumes that file name has MAXPATHLEN number of 
		characters allocated to it.  If go over limit,
		then truncate file name.  If delimeter is NULL
		use default file delimeter.
	
	RETURNS:
*******************************************************************/

void
filename_cat(basename, append_string, delimeter)
char *basename;		/* INPUT AND RETURNS */
char *append_string;
char *delimeter;
{
	int len_basename;

	if (delimeter == NULL) {
		delimeter = file_delimeter();
	}

	len_basename = strlen(basename);
	if (len_basename > MAXPATHLEN) {
		/* Over ran path -- minimize damage. */
		basename[MAXPATHLEN] = '\0';
		return;
	}

	if (len_basename + 1 > MAXPATHLEN) {
		/* Can not add even part of append_string */
		return;
	}
	strcat(basename, delimeter);

	if (len_basename + 1 + strlen(append_string) > MAXPATHLEN) {
		strncat(basename, append_string, MAXPATHLEN - len_basename - 1);
		basename[MAXPATHLEN] = '\0';
	}
	else {
		strcat(basename, append_string);
	}
}

/******************************************************************

	Takes buf then make a copy of it, allocating space if needed 
	If buf is zero lenth, then return NULL
 
******************************************************************/

char *
string(buf)
char *buf;
{
	char *str;
	int len;

	if ( buf == NULL ) {
		return ( (char *)NULL);
	}
	len = strlen(buf);
	if (len == 0)
		return ( (char *)NULL);
	str = (char *) new((len+1) * sizeof(char) );
	strcpy(str, buf);
	return (str);
}

/******************************************************************

	DESCRIPTION: Add the extension "ext" to the filename if an
		extension does not yet exists.  If it does exist,
		then replace the extension if replace == TRUE;
		In all cases allocated space for the string.

	RETURN: allocated file name string.
 
******************************************************************/

char *
FileNameExt(filename, ext, replace)
char *filename;
char *ext;
int replace;
{
	int len;
	int extension;
	int i;
	char *new_filename;
	char *delimeter;	/* only first character hold delimeter char */

	extension = FALSE;
	/* See if there is already an extenstion */
	len = strlen(filename);
	for (i = len - 1;  i >= 0;  --i) {
		if (filename[i] == '.') {
			extension = TRUE;
			break;
		}
		/* char for file delimeter character.  On does the file
		** delimeter can be either "/" or "\"
		*/
		delimeter = file_delimeter();
		if (filename[i] == '/' || filename[i] == delimeter[0]) {
			break;
		}
	}

	if (extension) {
		/* File extension already exists */
		if (replace) {
			new_filename = new((i+strlen(ext)+2)*sizeof(char));
			strncpy(new_filename, filename, i);
			new_filename[i] = '\0';
			strcat(new_filename, ext);

		}
		else {
			new_filename = string(filename);
		}
	}
	else {
		new_filename = new((len+strlen(ext)+2) *sizeof(char));
		strcpy(new_filename, filename);
		strcat(new_filename, ext);

	}
	return (new_filename);
}


/******************************************************************

	DESCRIPTION: Determine if the character c is a potential
		number.  Includes characters: 0-9, -, +, and "."

	RETURN: 
		TRUE - is a number string
		FALSE - is NOT a number string
 
******************************************************************/

int
IsNumberString(c)
char c;
{
	return(    isdigit(c) 
		|| c == '.' 
	        || c == '+'
	        || c == '-'
	      );
}
#if defined(DOS) || defined(HP300) 
/*******************************************************************

	DESCRIPTION:


     strcmp() compares  its  arguments  and  returns  an  integer
     greater  than,  equal to, or less than 0, according as s1 is
     lexicographically greater than, equal to, or less  than  s2.
     strncmp()  makes  the same comparison but compares at most n
     characters.   Two  additional  routines   strcasecmp()   and
     strncasecmp()  compare the strings and ignore differences in
     case. These routines assume the  ASCII  character  set  when
     equating lower and upper case characters.



*******************************************************************/
int
strncasecmp(s1,s2,n)
char *s1;
char *s2;
int n;
{
	char c1, c2;
	int i;

	if (s1 == NULL ) return (0);
	if (s2 == NULL ) return (0);

	i = 0;
	while ( *s1 != '\0' && i < n ) {
		/* Convert to upper case */
		c1 = *s1;
		if ( islower( c1 ) ) {
			c1 = toupper(c1);
		}
		c2 = *s2;
		if ( islower( c2 ) ) {
			c2 = toupper(c2);
		}

		if ( c2 == '\0' ) {
			return (1); /* s1 > s2 */
		}
		if ( c2 != c1 ) {
			if ( c1 > c2 ) {
				return (1);
			} else {
				return (-1);
			}
		}
		++s1;
		++s2;
		++i;
	}
	if ( i != n ) {
		if ( *s2 != '\0' ) {
			return (-1); /* s1 < s2 */
		}
	}
	return (0);
}

int
strcasecmp(s1,s2)
char *s1;
char *s2;
{
	if ( strlen(s1) < strlen(s2) ) {
		return(-1);
	}
	if ( strlen(s1) > strlen(s2) ) {
		return(1);
	}
	return (strncasecmp(s1,s2,strlen(s1)));
}
#endif /* DOS and HP300 */
